{--------------------------------------------------------------------------
Sprache    : Delphi 1.0
Unit       : db2ascii.pas
Version    : 1.02
Aenderungen: 0.9  - 27.10.1996 - Betaversion
             1.00 - 02.11.1996 - Fehlerbeseitigung
             1.01 - 05.11.1996 - Neue Ereignisse:
                                 - BeforeExport
                                 - AfterExport
                                 - BeforeHeaderClose
                                 - BeforeRecordClose
                                 Neue Methoden:
                                 - AppendString
                                 Aenderungen:
                                 - Anpassung der Namen der Aufzaehl-Typen
                                   an den Delphi-Standard (TFileType, TConversion)
             1.02 - 09.11.1996 - Fehlerbeseitigung
Aufgaben   : Export von Daten aus TTable oder TQuery-Datasets in verschiedene
             ASCII Formate
Hinweise   : Programme, die diese Komponente benutzen, muessen mit der
             Anweisung $X+ compiliert werden
Fehler     : Fehler bitte melden an: scheffczyk@verwaltung.uni-mainz.de
----------------------------------------------------------------------------
Komponente : TDB2Ascii
----------------------------------------------------------------------------
Properties : DataSet                : TDataSet     Datenquelle (TTable- oder
                                                   TQuery-Komponente)
             DisableDataSetControls : Boolean      Wenn 'true' wird die Methode
                                                   'DisableControl' (der Daten-
                                                   quelle) vor und die Methode
                                                   'EnableControl' (der Daten-
                                                   quelle)nach dem Export der
                                                   Daten aufgerufen
             FileType               : TFileType    Typ der zu erzeugenden ASCII-
                                                   Datei, 'ftFixed' fuer feste Feld,
                                                   bzw. Recordlaengen, 'ftVariable'
                                                   fuer durch ein Zeichen
                                                   getrennte Felder mit variabler
                                                   Laenge
             ExportOnlyVisibleFields: Boolean      Wenn 'true' werden nur Felder
                                                   exportiert, deren Eigenschaft
                                                   'Visible' den Wert 'true' hat
             FileName               : String       Name der zu erstellenden ASCII-
                                                   Datei
             CreateHeaders          : Boolean      Wenn 'true' wird die Property
                                                   'DisplayLabel' der Felder als
                                                   Feldname in die erste Zeile der
                                                   ASCII-Datei geschrieben
             QuoteNonStringFields   : Boolean      Wenn 'true' werden alle Feld-
                                                   inhalte in Anfuehrungen gesetzt
             QuoteStringFields      : Boolean      Wenn 'true' werden String-Feld-
                                                   inhalte in Anfuehrungen gesetzt
             InsertSepBeforeRecord  : Boolean      Wenn 'true' wird ein Feldsepa-
                                                   rationszeichen vor dem ersten
                                                   Feld eines Datensatzes
                                                   eingefuegt
             InsertSepAfterRecord   : Boolean      Wenn 'true' wird ein Feldsepa-
                                                   rationszeichen nach dem ersten
                                                   Feld eines Datensatzes
                                                   eingefuegt
             ReplaceSepInString     : Boolean      Wenn 'true' werden eventuell in
                                                   Feldinhalten enthaltene Separa-
                                                   tionszeichen durch ein Ersatz-
                                                   zeichen ersetzt
             ReplaceQuoteInString   : Boolean      Wenn 'true' werden eventuell in
                                                   Feldinhalten enthaltene Anfueh-
                                                   rungszeichen durch ein Ersatz-
                                                   zeichen ersetzt
             ReplaceReturnInString  : Boolean      Wenn 'true' werden eventuell in
                                                   Feldinhalten enthaltene Zeilen-
                                                   vorschubszeichen (CR/LF)durch
                                                   ein Ersatzzeichen ersetzt
             Char4QuoteReplace      : Char         Das Ersatzzeichen fuer even-
                                                   tuell in Feldinhalten vorhande-
                                                   ne Anfuehrungszeichen
             Char4SepReplace        : Char         Das Ersatzzeichen fuer even-
                                                   tuell in Feldinhalten vorhande-
                                                   ne Feldseparationszeichen
             Char4ReturnReplace     : Char         Das Ersatzzeichen fuer even-
                                                   tuell in Feldinhalten vorhande-
                                                   ne Zeilenvorschuebe
             Char4Quote             : Char         Das zu verwendene
                                                   Anfuehrungszeichen
             Char4Sep               : Char         Das zu verwendene
                                                   Separationszeichen
             Char4Fill              : Char         Das zu verwendene Zeichen zum
                                                   Auffuellen von Feldinhalten auf
                                                   eine feste Feldlaenge bei dem
                                                   'FileType' 'fixed'
----------------------------------------------------------------------------
Events     : BeforeFieldWrite       : TFieldCheck  Wird aufgerufen bevor ein
                                                   Feldinhalt in die ASCII-Datei
                                                   geschrieben wird
             BeforeHeaderFieldWrite : TFieldCheck  Wird aufgerufen bevor ein
                                                   Feldbezeicher in die ASCII-
                                                   Datei geschrieben wird (nur,
                                                   wenn die Property 'Header' auf
                                                   'true' gesetzt ist)
             OnFieldRead            : TFieldCheck  Wird aufgerufen wenn ein
                                                   Feldinhalt der Komponente von
                                                   einer Datenquelle uebergeben
                                                   wird
             OnHeaderFieldRead      : TFieldCheck  Wird aufgerufen wenn ein Feld-
                                                   bezeichner der Komponente von
                                                   einer Datenquelle uebergeben
                                                   wird (nur, wenn die Property
                                                   'Header' auf 'true' gesetzt
                                                   ist)
             FOnNewRecord           : TNotifyEvent Wird aufgerufen, wenn ein
                                                   neuer Datensatz exportiert
                                                   wird
             FBeforeHeaderClose     : TNotifyEvent Wird aufgerufen, nachdem der
                                                   letzte Feldname exportiert
                                                   wurde und BEVOR der
                                                   Header mit einem CR/LF
                                                   abgeschlossen wird
             FBeforeRecordClose     : TNotifyEvent Wird aufgerufen, nachdem das
                                                   letzte Feld eines Datensatzes
                                                   exportiert wurde und BEVOR der
                                                   Datensatz mit einem CR/LF
                                                   abgeschlossen wird
             FBeforeExport          : TNotifyEvent Wird aufgerufen bevor der erste
                                                   Datensatz (oder die Feldnamen,
                                                   wenn die Property 'Header' auf
                                                   'true' gesetzt ist) exportiert
                                                   wird
             FAfterExport           : TNotifyEvent Wird aufgerufen nachdem der
                                                   letzte Datensatz exportiert
                                                   wurde
----------------------------------------------------------------------------
Methoden   : Create(aOwner: TComponent)            Erzeugt die Komponente wobei
                                                   aOwner den Besitzer der
                                                   Komponente bezeichnet
             Free                                  Ruft die Methode 'Destroy'
                                                   auf, wenn die Komponente zuvor
                                                   mit 'Create' erzeugt wurde
             Destroy                               Loescht die Komponente -
                                                   Achtung wenn die Komponente
                                                   noch nicht erzeugt oder schon
                                                   geloescht wurde fuehrt diese
                                                   Methode (im Gegensatz zu 'Free'
                                                   zu einem Laufzeitfehler
             Execute                               Startet den Export der Daten,
                                                   der Export terminiert automa-
                                                   tisch bei Erreichen des
                                                   letzten Datensatzes der Daten-
                                                   quelle
             AppendString                          Ermoeglicht es, innerhalb einer
                                                   UDP, die einem Ereignis dieser
                                                   Komponente zugeordnet wurde,
                                                   zusaetzliche Strings in die
                                                   Ausgabedatei einzufuegen
---------------------------------------------------------------------------}
unit db2ascii;

{$X+}
{==========================================================================}
interface

uses
  Classes, SysUtils, WinTypes, WinProcs, DB, DBTables, Dialogs, Forms;

type
  TFileType      = (ftVariable,ftFixed);
  TConversion    = (ccNone,ccAscii2Ansi,ccAnsi2Ascii);
  TFileState     = (fsOpen,fsClose);
  TFieldCheck    = procedure (FieldName: String; VAR WriteString: String)
                   of Object;
  TDB2Ascii   = class(TComponent)
  private
    { Private declarations }
    FBeforeFieldWrite       : TFieldCheck;
    FBeforeHeaderFieldWrite : TFieldCheck;
    FOnFieldRead            : TFieldCheck;
    FOnHeaderFieldRead      : TFieldCheck;
    FOnNewRecord            : TNotifyEvent;
    FBeforeHeaderClose      : TNotifyEvent;
    FBeforeRecordClose      : TNotifyEvent;
    FBeforeExport           : TNotifyEvent;
    FAfterExport            : TNotifyEvent;
    FDataSet                : TDataSet;
    FDisableDataSetControls : Boolean;
    FFileType               : TFileType;
    FCharSetConversion      : TConversion;
    FAsciiOutputFile        : TextFile;
    FAsciiOutputFileState   : TFileState;
    FFileName               : String;
    FChar4Quote             : Char;
    FChar4QuoteReplace      : Char;
    FChar4Sep               : Char;
    FChar4SepReplace        : Char;
    FChar4ReturnReplace     : Char;
    FChar4Fill              : Char;
    FExportOnlyVisibleFields: Boolean;
    FCreateHeaders          : Boolean;
    FQuoteNonString         : Boolean;
    FQuoteString            : Boolean;
    FSepBeforeRecord        : Boolean;
    FSepAfterRecord         : Boolean;
    FReplaceSepInString     : Boolean;
    FReplaceQuoteInString   : Boolean;
    FReplaceReturnInString  : Boolean;
    procedure ExportData;
    procedure WriteHeader( Field      : TField;
                           oldChars   : String;
                           newChars   : String);
    procedure WriteField(  Field      : TField;
                           oldChars   : String;
                           newChars   : String);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy;
    procedure Execute;
    procedure AppendString(aString: String; var Done: Boolean);
  published
    { Published declarations }
{--events----------------------------------------------------------------}
    property BeforeFieldWrite       : TFieldCheck  read  FBeforeFieldWrite
                                                   write FBeforeFieldWrite;
    property BeforeHeaderFieldWrite : TFieldCheck  read  FBeforeHeaderFieldWrite
                                                   write FBeforeHeaderFieldWrite;
    property OnFieldRead            : TFieldCheck  read  FOnFieldRead
                                                   write FOnFieldRead;
    property OnHeaderFieldRead      : TFieldCheck  read  FOnHeaderFieldRead
                                                   write FOnHeaderFieldRead;
    property OnNewRecord            : TNotifyEvent read  FOnNewRecord
                                                   write FOnNewRecord;
    property BeforeHeaderClose      : TNotifyEvent read  FBeforeHeaderClose
                                                   write FBeforeHeaderClose;
    property BeforeRecordClose      : TNotifyEvent read  FBeforeRecordClose
                                                   write FBeforeRecordClose;
    property BeforeExport           : TNotifyEvent read  FBeforeExport
                                                   write FBeforeExport;
    property AfterExport            : TNotifyEvent read  FAfterExport
                                                   write FAfterExport;
{--properties------------------------------------------------------------}
    property DataSet                : TDataSet     read    FDataSet
                                                   write   FDataSet;
    property DisableDataSetControls : Boolean      read    FDisableDataSetControls
                                                   write   FDisableDataSetControls;
    property ExportOnlyVisibleFields: Boolean      read    FExportOnlyVisibleFields
                                                   write   FExportOnlyVisibleFields;
    property FileType               : TFileType    read    FFileType
                                                   write   FFileType;
    property CharSetConversion      : TConversion  read    FCharSetConversion
                                                   write   FCharSetConversion;
    property FileName               : String       read    FFileName
                                                   write   FFileName;
    property CreateHeaders          : Boolean      read    FCreateHeaders
                                                   write   FCreateHeaders;
    property QuoteNonStringFields   : Boolean      read    FQuoteNonString
                                                   write   FQuoteNonString;
    property QuoteStringFields      : Boolean      read    FQuoteString
                                                   write   FQuoteString;
    property InsertSepBeforeRecord  : Boolean      read    FSepBeforeRecord
                                                   write   FSepBeforeRecord;
    property InsertSepAfterRecord   : Boolean      read    FSepAfterRecord
                                                   write   FSepAfterRecord;
    property ReplaceSepInString     : Boolean      read    FReplaceSepInString
                                                   write   FReplaceSepInString;
    property ReplaceQuoteInString   : Boolean      read    FReplaceQuoteInString
                                                   write   FReplaceQuoteInString;
    property ReplaceReturnInString  : Boolean      read    FReplaceReturnInString
                                                   write   FReplaceReturnInString;
    property Char4QuoteReplace      : Char         read    FChar4QuoteReplace
                                                   write   FChar4QuoteReplace;
    property Char4SepReplace        : Char         read    FChar4SepReplace
                                                   write   FChar4SepReplace;
    property Char4Quote             : Char         read    FChar4Quote
                                                   write   FChar4Quote;
    property Char4ReturnReplace     : Char         read    FChar4ReturnReplace
                                                   write   FChar4ReturnReplace;
    property Char4Sep               : Char         read    FChar4Sep
                                                   write   FChar4Sep;
    property Char4Fill              : Char         read    FChar4Fill
                                                   write   FChar4Fill;
  end;

procedure Register;

{==========================================================================}
implementation

uses
    _4passtr;

{==========================================================================}
{ Implementation der als unit-spezifischen Prozeduren und Funktionen       }
{==========================================================================}

{--------------------------------------------------------------------------
Name      : Register
Typ       : Prozedur
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 21.10.1995
Autor     : Scheffczyk
Aufgabe   : Registriert die Komponente in der Komponentenpalette der IDE
Parameter : -
Rueckgabe : -
Fehler    : -
Hinweise  : Die Komponente wird auf der Registerseite 'DBAddons' installiert
---------------------------------------------------------------------------}
procedure Register;
begin
     RegisterComponents('DBAddons', [TDB2Ascii]);
end;

{==========================================================================}
{ Implementation der als public deklarierten Prozeduren und Funktionen     }
{==========================================================================}

{--------------------------------------------------------------------------
Name      : TDB2Ascii.Create
Typ       : Constructor
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 21.10.1995
Autor     : Scheffczyk
Aufgabe   : Erzeugt die Komponente und initialisiert alle Eigenschaften und
            Ereignishandler
Parameter : AOwner: TComponent : Besitzer der Komponente, normalerweise die
                                 Form, auf der die Komponente eingesetzt wird
Rueckgabe : -
Fehler    : -
Hinweise  : Die Eigenschaften 'FChar4ReturnReplace' und 'FChar4Fill' werden
            mit einem Leerzeichen initialisiert. Der Objektinspektor zeigt
            Leerzeichen als Wert einer Char-Eigenschaft als '#32' dar, um sie
            von einem leeren Wert unterscheiden zu koennen
---------------------------------------------------------------------------}
constructor TDB2Ascii.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);
     FBeforeFieldWrite       := nil;
     FBeforeHeaderFieldWrite := nil;
     FOnFieldRead            := nil;
     FOnHeaderFieldRead      := nil;
     FDataSet                := nil;
     FAsciiOutputFileState   := fsClose;
     FFileType               := ftVariable;
     FCharSetConversion      := ccNone;
     FFileName               := '';
     FDisableDataSetControls := False;
     FCreateHeaders          := False;
     FQuoteNonString         := False;
     FQuoteString            := False;
     FSepBeforeRecord        := False;
     FSepAfterRecord         := False;
     FExportOnlyVisibleFields:= True;
     FReplaceSepInString     := True;
     FReplaceQuoteInString   := True;
     FReplaceReturnInString  := True;
     FChar4Quote             := '"';
     FChar4QuoteReplace      := '^';
     FChar4Sep               := ';';
     FChar4SepReplace        := ',';
     FChar4ReturnReplace     := ' ';
     FChar4Fill              := ' ';
end;

{--------------------------------------------------------------------------
Name      : TDB2Ascii.Destroy
Typ       : Destructor
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 21.10.1995
Autor     : Scheffczyk
Aufgabe   : Gibt alle bei der Erstellung der Komponente belegten Resourcen
            wieder frei
Parameter : -
Rueckgabe : -
Fehler    : -
Hinweise  : -
---------------------------------------------------------------------------}
destructor TDB2Ascii.Destroy;
begin
     FBeforeFieldWrite       := nil;
     FBeforeHeaderFieldWrite := nil;
     FOnFieldRead            := nil;
     FOnHeaderFieldRead      := nil;
     FDataSet                := nil;
     inherited Destroy;
end;

{--------------------------------------------------------------------------
Name      : TDB2Ascii.Execute
Typ       : Prozedur
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 21.10.1995
Autor     : Scheffczyk
Aufgabe   : Der Aufruf dieser Prozedur startet den Export der Daten, wenn ein
            gueltiges DataSet und ein gueltiger Dateiname fuer die ASCII-Datei
            angegeben wurde, ansonster werden entsprechende Fehlermeldungen
            ausgegeben
Parameter : -
Rueckgabe : -
Fehler    : -
Hinweise  : Oeffnet und schliesst die ASCII-Datei und ruft die private
            Prozedur 'ExportData' auf
---------------------------------------------------------------------------}
Procedure TDB2Ascii.Execute;
begin
     if FFileName = '' then
     begin
          MessageDLG('Property FileName not set',mtError,[mbOk],0);
     end
     else
     begin
          if FDataSet = nil then
          begin
                MessageDLG('Property DataSet not set',mtError,[mbOk],0);
          end
          else
          begin
               AssignFile(FAsciiOutputFile,FFileName);
               Rewrite(FAsciiOutputFile);
               FAsciiOutputFileState := fsOpen;
               {---------------------------------------------}
               { Temporaeres Loesen der Datenquelle von      }
               { allen Datasource-Objekten zum Beschleunigen }
               { des Exportvorganges, wenn der Eigenschaft   }
               { 'FDisableDataSetControls' der Wert 'true'   }
               { zugewiesen wurde.                           }
               {---------------------------------------------}
               if FDisableDataSetControls then
               begin
                    FDataSet.DisableControls;
               end;
               {---------------------------------------------}
               { Wenn zugeordnet, Aufruf der UDP fuer das    }
               { Ereignis 'BeforeExport'                     }
               {---------------------------------------------}
               if Assigned(FBeforeExport) then
               begin
                    FBeforeExport(self);
               end;
               {---------------------------------------------}
               { Aufruf der eigentlichen Exportfunktion      }
               {---------------------------------------------}
               ExportData;
               {---------------------------------------------}
               { Wenn zugeordnet, Aufruf der UDP fuer das    }
               { Ereignis 'AfterExport'                      }
               {---------------------------------------------}
               if Assigned(FAfterExport) then
               begin
                    FAfterExport(self);
               end;
               {---------------------------------------------}
               { Wiederherstellung der Verbindungen der      }
               { Datenquelle zu Datasource-Objekten, wenn    }
               { diese Verbindungen vor dem Export geloest   }
               { wurden                                      }
               {---------------------------------------------}
               if FDisableDataSetControls then
               begin
                    FDataSet.DisableControls;
               end;
               CloseFile(FAsciiOutputFile);
               FAsciiOutputFileState := fsClose;
          end;
     end;
end;

{--------------------------------------------------------------------------
Name      : TDB2Ascii.AppendString
Typ       : Prozedur
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 05.11.1995
Autor     : Scheffczyk
Aufgabe   : Erlaubt das Einfuegen von Strings waehrend des Exports der Daten
Parameter :     aString : String   : Der einzufuegende String
            var Done    : Boolean  : 'true, wenn der String eingefuegt werden
                                     konnte, ansonsten 'false'
Rueckgabe : -
Fehler    : -
Hinweise  : Diese Funktion wird normalerweise in einer UDF aufgerufen, die
            einem Ereigneis der Komponente zugewiesen wurde.
            Das Einfuegen ist nur waehrend des Exportes moeglich, d.h. nachdem
            die Methode 'Execute' aufgerufen wurde
---------------------------------------------------------------------------}
procedure TDB2Ascii.AppendString(aString: String; var Done: Boolean);
begin
     if FAsciiOutputFileState = fsOpen then
     begin
          write(FAsciiOutputFile,aString);
          Done := true;
     end
     else
     begin
          Done := false;
     end;
end;

{==========================================================================}
{ Implementation der als protected deklarierten Prozeduren und Funktionen  }
{==========================================================================}

{==========================================================================}
{ Implementation der als privat deklarierten Prozeduren und Funktionen     }
{==========================================================================}

{--------------------------------------------------------------------------
Name      : TDB2Ascii.ExportData
Typ       : Prozedur
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 21.10.1995
Autor     : Scheffczyk
Aufgabe   : Durchlaufen des DataSets vom ersten bis zum letzten Datensatz
            und fuer jeden Datensatz Export aller Felder, deren Eigenschaft
            'visible' auf 'true' gesetzt ist. Teilauswertung der Eigenschaften
            der Komponente
Parameter : -
Rueckgabe : -
Fehler    : -
Hinweise  : -
---------------------------------------------------------------------------}
procedure TDB2Ascii.ExportData;
var
   FirstField : Boolean;
   i          : Integer;
   oldChars   : String;
   newChars   : String;
begin
     oldChars := '';
     newChars := '';
     {-------------------------------------------------------}
     { Abfrage der Eigenschaften fuer das Austausch von      }
     { Separator- und Anfuehrungszeichen in Strings          }
     { Der eigentliche Austausch der Zeichen erfolgt in den  }
     { Prozeduren 'WriteHeader' und 'WriteField'             }
     {-------------------------------------------------------}
     if FReplaceSepInString then
     begin
          oldChars := FChar4Sep;
          newChars := FChar4SepReplace;
     end;
     if FReplaceQuoteInString then
     begin
          oldChars := oldChars + FChar4Quote;
          newChars := newChars + FChar4QuoteReplace;
     end;
     {=======================================================}
     { Abfrage der Eigenschaft 'CreateHeader' fuer das       }
     { Voranstellen der Eigenschaft 'DisplayLabel' der       }
     { Felder als eigener Datensatz vor den eigentlichen     }
     { Datensaetzen der Datenquelle                          }
     {=======================================================}
     if FCreateHeaders = True then
     begin
          {--------------------------------------------------}
          { Einfuegen eines Separators vor dem ersten Label, }
          { wenn die entsprechende Eigenschaft gesetzt ist   }
          {--------------------------------------------------}
          if FSepBeforeRecord = true then
          begin
               write(FAsciiOutputFile,FChar4Sep);
          end;
          FirstField := True;
          {--------------------------------------------------}
          { Schleife zum Durchlaufen der Felder eines        }
          { Datensatzes                                      }
          {--------------------------------------------------}
          for i := 0 to FDataSet.FieldCount-1 do
          begin
               {---------------------------------------------}
               { Ausgabe eines Labels nur, wenn die Eigen-   }
               { schaft 'Visible' des zugehoerigen Feldes    }
               { den Wert 'true' hat oder die Eigenschaft    }
               { 'ExportOnlyVisibleFields' der Komponente    }
               { der Wert 'false' zugewiesen wurde           }
               {---------------------------------------------}
               if ((FDataSet.Fields[i].Visible) or
                   (ExportOnlyVisibleFields = false)) then
               begin
                    {----------------------------------------}
                    { Bei Dateityp 'variable' vor jedem      }
                    { Feld, ausser vor dem ersten, ein       }
                    { Separatorzeichen einfuegen             }
                    {----------------------------------------}
                    if ((not FirstField) and (FFileType = ftVariable)) then
                    begin
                         write(FAsciiOutputFile,FChar4Sep);
                    end;
                    WriteHeader( FDataSet.Fields[i],
                                 oldChars,
                                 newChars);
                    FirstField := false;
               end;
          end;
          {--------------------------------------------------}
          { Einfuegen eines Separators nach dem letzten      }
          { Label, wenn die entsprechende Eigenschaft        }
          { gesetzt ist                                      }
          {--------------------------------------------------}
          if FSepAfterRecord = true then
          begin
               write(FAsciiOutputFile,FChar4Sep);
          end;
          {--------------------------------------------------}
          { Wenn zugeordnet, Aufruf der UDP fuer das         }
          { Ereignis 'BeforeHeaderClose'                     }
          {--------------------------------------------------}
          if Assigned(FBeforeHeaderClose) then
          begin
               FBeforeHeaderClose(self);
          end;
          {--------------------------------------------------}
          { Abschluss des Header-Datensatzes durch CR/LF     }
          {--------------------------------------------------}
          Writeln (FAsciiOutputFile);
     end;
     FDataSet.First;
     {=======================================================}
     { Schleife zum Durchlaufen aller vorhandenen            }
     { Datensaetze der Datenquelle                           }
     {=======================================================}
     while not(FDataSet.Eof) do
     begin
          {--------------------------------------------------}
          { Wenn zugeordnet, Aufruf der UDP fuer das         }
          { Ereignis 'OnNewRecord'                           }
          {--------------------------------------------------}
          if Assigned(FOnNewRecord) then
          begin
               FOnNewRecord(Self);
          end;
          Application.ProcessMessages;
          {--------------------------------------------------}
          { Einfuegen eines Separators vor dem ersten Feld,  }
          { wenn die entsprechende Eigenschaft gesetzt ist   }
          {--------------------------------------------------}
          if FSepBeforeRecord = true then
          begin
               write(FAsciiOutputFile,FChar4Sep);
          end;
          FirstField := True;
          {--------------------------------------------------}
          { Schleife zum Durchlaufen der Felder eines        }
          { Datensatzes                                      }
          {--------------------------------------------------}
          for i := 0 to FDataSet.FieldCount-1 do
          begin
               {---------------------------------------------}
               { Ausgabe eines Feldes nur, wenn dessen       }
               { Eigenschaft 'Visible' den Wert 'true' hat   }
               { oder die Eigenschaft                        }
               { 'ExportOnlyVisibleFields' der Komponente    }
               { der Wert 'false' zugewiesen wurde           }
               {---------------------------------------------}
               if ((FDataSet.Fields[i].Visible) or
                   (ExportOnlyVisibleFields = false)) then
               begin
                    {----------------------------------------}
                    { Bei Dateityp 'variable' vor jedem      }
                    { Feld, ausser vor dem ersten, ein       }
                    { Separatorzeichen einfuegen             }
                    {----------------------------------------}
                    if ((not FirstField) and (FFileType = FTVariable)) then
                    begin
                         write(FAsciiOutputFile,FChar4Sep);
                    end;
                    WriteField( FDataSet.Fields[i],
                                oldChars,
                                newChars);
                    FirstField := false;
               end;
          end;
          {--------------------------------------------------}
          { Einfuegen eines Separators nach dem letzten Feld,}
          { wenn die entsprechende Eigenschaft gesetzt ist   }
          {--------------------------------------------------}
          if FSepAfterRecord = true then
          begin
               write(FAsciiOutputFile,FChar4Sep);
          end;
          {--------------------------------------------------}
          { Wenn zugeordnet, Aufruf der UDP fuer das         }
          { Ereignis 'BeforeRecordClose'                     }
          {--------------------------------------------------}
          if Assigned(FBeforeRecordClose) then
          begin
               FBeforeRecordClose(self);
          end;
          {--------------------------------------------------}
          { Abschluss des Datensatzes durch CR/LF            }
          {--------------------------------------------------}
          Writeln (FAsciiOutputFile);
          FDataSet.Next;
     end;
 end;

{--------------------------------------------------------------------------
Name      : TDB2Ascii.WriteHeader
Typ       : Prozedur
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 21.10.1995
Autor     : Scheffczyk
Aufgabe   : Export eines Feldlabels in die ASCII-Datei
Parameter : Field    : TField : Das Feld, dessen Inhalt in die ASCII-Datei
                                geschrieben werden soll
            oldChars : String : Die in dem Feldinhalt zu erzetzenden Zeichen
            newChars : String : Die Ersatzzeichen
Rueckgabe : -
Fehler    : -
Hinweise  : 'oldChars' enthaelt die gewaehlten Separations- und Anfuehrungs-
            zeichen und 'newChars' die angegebenen Ersatzzeichen wenn die
            entsprechenden Eigenschaften der Komponente den Austausch an-
            ordnen
            Die Feldlabel werden prinzipiell wie String-Felder behandelt
---------------------------------------------------------------------------}
procedure TDB2Ascii.WriteHeader( Field      : TField;
                                 oldChars   : String;
                                 newChars   : String);
var
   aValue : String;
begin
     aValue := Field.DisplayLabel;
     {--------------------------------------------------------}
     { Wenn zugeordnet, Aufruf der UDP fuer das Ereignis      }
     { 'OnHeaderFieldRead'                                    }
     {--------------------------------------------------------}
     if Assigned(FOnHeaderFieldRead) then
     begin
          FOnHeaderFieldRead(Field.FieldName,aValue);
     end;
     {--------------------------------------------------}
     { Ersetzen eventuell im Feldinhalt vorhandener     }
     { Separations- und Anfuehrungszeichen              }
     {--------------------------------------------------}
     aValue := ReplaceStr(aValue,OldChars,NewChars);
     {--------------------------------------------------}
     { Ersetzen eventuell im Feldlabel vorhandener      }
     { Zeilenumbrueche, wenn die entsprechende Eigen-   }
     { schaft auf 'true' gesetzt ist                    }
     {--------------------------------------------------}
     if FReplaceReturnInString then
     begin
          aValue := ReplaceSubStr(aValue,#13+#10,FChar4ReturnReplace);
     end;
     {--------------------------------------------------}
     { Bei AusgabeTyp 'fixed' den Feldinhalt auf die    }
     { Laenge auffuellen bzw. kuerzen, die als Wert     }
     { 'DisplayWidth' des Feldes angegeben ist          }
     {--------------------------------------------------}
     if FFileType = ftFixed then
     begin
          aValue := LeftStr( RFill( aValue,
                                    Field.DisplayWidth,
                                    FChar4Fill),
                             Field.DisplayWidth);
     end;
     {--------------------------------------------------}
     { Umschliessen des Feldinhaltes mit Anfuehrungs-   }
     { zeichen, wenn die entsprechende Eigenschaft      }
     { auf 'true' gesetzt ist                           }
     {--------------------------------------------------}
     if FQuoteString then
     begin
          aValue := FChar4Quote +
                    aValue +
                    FChar4Quote;
     end;
    {--------------------------------------------------------}
    { Wenn zugeordnet, Aufruf der UDP fuer das Ereignis      }
    { 'BeforeHeaderFieldWrite'                                          }
    {--------------------------------------------------------}
    if Assigned(FBeforeHeaderFieldWrite) then
    begin
         FBeforeHeaderFieldWrite(Field.FieldName,aValue);
    end;
    {--------------------------------------------------------}
    { Konvertierung des Zeichensatzes gemaess dem Wert       }
    { der entsprechenden Eigenschaft                         }
    {--------------------------------------------------------}
    case FCharSetConversion of
         ccAnsi2Ascii: aValue := ccsAnsi2Ascii(aValue);
         ccAscii2Ansi: aValue := ccsAscii2Ansi(aValue);
    end;
    {--------------------------------------------------------}
    { Ausgabe des bearbeiteten Feldlabels in die             }
    { ASCII-Datei                                            }
    {--------------------------------------------------------}
    Write(FAsciiOutputFile,aValue);
end;

{--------------------------------------------------------------------------
Name      : TDB2Ascii.WriteField
Typ       : Prozedur
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 21.10.1995
Autor     : Scheffczyk
Aufgabe   : Export eines Feldinhaltes in die ASCII-Datei
Parameter : Field    : TField : Das Feld, dessen Inhalt in die ASCII-Datei
                                geschrieben werden soll
            oldChars : String : Die in dem Feldinhalt zu erzetzenden Zeichen
            newChars : String : Die Ersatzzeichen
Rueckgabe : -
Fehler    : -
Hinweise  : 'oldChars' enthaelt die gewaehlten Separations- und Anfuehrungs-
            zeichen und 'newChars' die angegebenen Ersatzzeichen wenn die
            entsprechenden Eigenschaften der Komponente den Austausch an-
            ordnen
---------------------------------------------------------------------------}
procedure TDB2Ascii.WriteField( Field      : TField;
                                oldChars   : String;
                                newChars   : String);
var
   aValue : String;
begin
     aValue := Field.DisplayText;
     {--------------------------------------------------------}
     { Wenn zugeordnet, Aufruf der UDP fuer das Ereignis      }
     { 'OnFieldRead'                                          }
     {--------------------------------------------------------}
     if Assigned(FOnFieldRead) then
     begin
          FOnFieldRead(Field.FieldName,aValue);
     end;
     {--------------------------------------------------------}
     { Abfrage des Feldtypes und Ausfuehrung der entsprechen- }
     { den Routinen                                           }
     {--------------------------------------------------------}
     Case Field.DataType of
     ftUnknown,
     ftString,
     ftDate,
     ftTime,
     ftDateTime :
     begin
          {--------------------------------------------------}
          { Ersetzen eventuell im Feldinhalt vorhandener     }
          { Separations- und Anfuehrungszeichen              }
          {--------------------------------------------------}
          aValue := ReplaceStr(aValue,OldChars,NewChars);
          {--------------------------------------------------}
          { Ersetzen eventuell im Feldinhalt vorhandener     }
          { Zeilenumbrueche, wenn die entsprechende Eigen-   }
          { schaft auf 'true' gesetzt ist                    }
          {--------------------------------------------------}
          if FReplaceReturnInString then
          begin
               aValue := ReplaceSubStr(aValue,#13+#10,FChar4ReturnReplace);
          end;
          {--------------------------------------------------}
          { Bei AusgabeTyp 'fixed' den Feldinhalt auf die    }
          { Laenge auffuellen bzw. kuerzen, die als Wert     }
          { 'DisplayWidth' des Feldes angegeben ist          }
          {--------------------------------------------------}
          if FFileType = ftFixed then
          begin
               aValue := LeftStr( RFill( aValue,
                                         Field.DisplayWidth,
                                         FChar4Fill),
                                  Field.DisplayWidth);
          end;
          {--------------------------------------------------}
          { Umschliessen des Feldinhaltes mit Anfuehrungs-   }
          { zeichen, wenn die entsprechende Eigenschaft      }
          { auf 'true' gesetzt ist                           }
          {--------------------------------------------------}
          if FQuoteString then
          begin
               aValue := FChar4Quote +
                         aValue +
                         FChar4Quote;
          end;
     end;
     ftSmallInt,
     ftInteger,
     ftWord,
     ftBoolean,
     ftFloat,
     ftCurrency,
     ftBCD:
     begin
          {--------------------------------------------------}
          { Bei AusgabeTyp 'fixed' den Feldinhalt auf die    }
          { Laenge auffuellen bzw. kuerzen, die als Wert     }
          { 'DisplayWidth' des Feldes angegeben ist          }
          {--------------------------------------------------}
          if FFileType = ftFixed then
          begin
               aValue := LeftStr( RFill( aValue,
                                         Field.DisplayWidth,
                                         FChar4Fill),
                                  Field.DisplayWidth);
          end;
          {--------------------------------------------------}
          { Umschliessen des Feldinhaltes mit Anfuehrungs-   }
          { zeichen, wenn die entsprechende Eigenschaft      }
          { auf 'true' gesetzt ist                           }
          {--------------------------------------------------}
          if FQuoteNonString then
          begin
               aValue := FChar4Quote +
                         aValue +
                         FChar4Quote;
          end;
     end;
     ftBytes,
     ftVarBytes,
     ftBlob,
     ftMemo,
     ftGraphic :
     begin
          {--------------------------------------------------}
          { Bei AusgabeTyp 'fixed' den Feldinhalt auf die    }
          { Laenge auffuellen bzw. kuerzen, die als Wert     }
          { 'DisplayWidth' des Feldes angegeben ist          }
          {--------------------------------------------------}
          if FFileType = ftFixed then
          begin
               aValue := LeftStr( RFill( aValue,
                                         Field.DisplayWidth,
                                         FChar4Fill),
                                  Field.DisplayWidth);
          end;
          {--------------------------------------------------}
          { Umschliessen des Feldinhaltes mit Anfuehrungs-   }
          { zeichen, wenn die entsprechende Eigenschaft      }
          { auf 'true' gesetzt ist                           }
          {--------------------------------------------------}
          if FQuoteNonString then
          begin
               aValue := FChar4Quote +
                         aValue +
                         FChar4Quote;
          end;
     end;
    end;
    {--------------------------------------------------------}
    { Wenn zugeordnet, Aufruf der UDP fuer das Ereignis      }
    { 'BeforeFieldWrite'                                     }
    {--------------------------------------------------------}
    if Assigned(FBeforeFieldWrite) then
    begin
         FBeforeFieldWrite(Field.FieldName,aValue);
    end;
    {--------------------------------------------------------}
    { Konvertierung des Zeichensatzes gemaess dem Wert       }
    { der entsprechenden Eigenschaft                         }
    {--------------------------------------------------------}
    case FCharSetConversion of
         ccAnsi2Ascii: aValue := ccsAnsi2Ascii(aValue);
         ccAscii2Ansi: aValue := ccsAscii2Ansi(aValue);
    end;
    {--------------------------------------------------------}
    { Ausgabe des bearbeiteten Feldinhaltes in die           }
    { ASCII-Datei                                            }
    {--------------------------------------------------------}
    Write(FAsciiOutputFile,aValue);
end;

end.
